@@ -1,3 +1,4 @@
+import sys
 
 from antlr4 import InputStream, CommonTokenStream
 
@@ -26,6 +27,10 @@ class DrbXQuery:
         self.tree = self.parser.module()
 
     def execute(self, node: DrbNode, external_var: dict = None):
+
+        old_limit = sys.getrecursionlimit()
+        sys.setrecursionlimit(10000)
+
         # Execute the query on the node
         visitor = DrbQueryVisitor(DynamicContext(node), tokens=self.stream)
         visitor.external_var_map = external_var
@@ -34,4 +39,7 @@ class DrbXQuery:
         output = visitor.visitModule(self.tree)
         if not isinstance(output, list):
             output = [output]
+
+        sys.setrecursionlimit(old_limit)
+
         return output
