@@ -7,7 +7,9 @@ from drb.factory import DrbFactoryResolver
 
 from drb_xquery.drb_xquery_context import DynamicContext, StaticContext
 import re
+from dateutil.parser import parse
 
+from drb_xquery.drb_xquery_item import DrbXqueryItem
 from drb_xquery.drb_xquery_utils import DrbQueryFuncUtil
 from drb_xquery.execptions import DynamicException, ErrorXQUERY
 
@@ -179,7 +181,10 @@ def get_last(func, current_node, static_context):
 
 
 def count(func, current_node, static_context, elt_to_count):
-    return len(elt_to_count)
+    if isinstance(elt_to_count, list):
+        return len(elt_to_count)
+    else:
+        return 1
 
 
 def get_position(func, current_node, static_context):
@@ -192,8 +197,23 @@ def get_name(func, current_node, static_context, elt=None):
     if elt is None:
         elt = current_node
     elif isinstance(elt, list):
+        if len(elt) == 0:
+            return ''
         elt = elt[0]
 
+    if isinstance(elt, DynamicContext):
+        if elt.is_node():
+            elt = elt.node
+
+    if isinstance(elt, DrbXqueryItem):
+        if elt.prefix is not None and len(elt.prefix) > 0:
+            return elt.prefix + ':' + elt.name
+    elif isinstance(elt, (DrbNode, DynamicContext)):
+        namespace = elt.namespace_uri
+        if namespace is not None and len(namespace) > 0:
+            ns = static_context.get_namespace_prefix_name(namespace)
+            if ns is not None and len(ns) > 0:
+                return ns + ':' + elt.name
     return elt.name
 
 
@@ -253,8 +273,10 @@ def ends_with(func, current_node, static_context, *args):
 def contains(func, current_node, static_context, *args):
     check_arg(func, 2, *args)
 
-    if is_elt_empty(args[0]) or is_elt_empty(args[1]):
+    if is_elt_empty(args[0]):
         return False
+    if is_elt_empty(args[1]):
+        return True
     str1 = DrbQueryFuncUtil.get_string(args[0])
     str2 = DrbQueryFuncUtil.get_string(args[1])
 
@@ -271,7 +293,7 @@ def substring(func, current_node, static_context, *args):
 
     if isinstance(start, float) and math.isnan(start):
         return ''
-    if start >= len(str1):
+    if start > len(str1):
         return ''
     if isinstance(start, float) and start == float('+inf'):
         return ''
@@ -388,7 +410,12 @@ def string_concat(func, current_node, static_context, *args):
 
 
 def string_compare(func, current_node, static_context, *args):
-    check_arg(func, 2, *args)
+    check_arg_min_max(func, 2, 3, *args)
+    if len(args) == 3:
+        raise DynamicException(ErrorXQUERY.FOCH0002,
+                               " func " + func +
+                               " not support collation")
+
     if is_empty_list(args[0]) or is_empty_list(args[1]):
         return []
 
@@ -410,6 +437,26 @@ def string_compare(func, current_node, static_context, *args):
     return 0
 
 
+def lower_case(func, current_node, static_context, *args):
+    check_arg(func, 1, *args)
+    if is_empty_list(args[0]):
+        return []
+
+    str1 = DrbQueryFuncUtil.get_string(args[0])
+
+    return str1.lower()
+
+
+def upper_case(func, current_node, static_context, *args):
+    check_arg(func, 1, *args)
+    if is_empty_list(args[0]):
+        return []
+
+    str1 = DrbQueryFuncUtil.get_string(args[0])
+
+    return str1.lower()
+
+
 def string_tokenize(func, current_node, static_context, *args):
     check_arg_min_max(func, 1, 2, *args)
     if is_empty_list(args[0]):
@@ -526,6 +573,101 @@ def empty(func, current_node, static_context, *args):
     return False
 
 
+def reverse(func, current_node, static_context, *args):
+    check_arg(func, 1, *args)
+
+    if isinstance(args[0], list):
+        args[0].reverse()
+
+    return args[0]
+
+
+def distinct_value(func, current_node, static_context, *args):
+    check_arg(func, 1, *args)
+
+    if isinstance(args[0], list):
+        list_uniq_values = []
+
+        for item in args[0]:
+            value = DrbQueryFuncUtil.get_value(item)
+            if value not in list_uniq_values:
+                list_uniq_values.append(value)
+        return list_uniq_values
+
+    return DrbQueryFuncUtil.get_value(args[0])
+
+
+def deep_equal(func, current_node, static_context, *args):
+    check_arg(func, 2, *args)
+
+    if isinstance(args[0], list):
+        if not isinstance(args[1], list):
+            return False
+
+        if len(args[0]) != len(args[1]):
+            return False
+        for i in range(len(args[0])):
+            if not deep_equal(func, current_node,
+                              static_context,
+                              args[0][i], args[1][i]):
+                return False
+        return True
+
+    left_value, right_value = \
+        DrbQueryFuncUtil.convert_before_compare(args[0],
+                                                args[1])
+    if not DrbQueryFuncUtil.is_equal(left_value, right_value):
+        return False
+
+    node1 = args[0]
+    node2 = args[1]
+
+    if isinstance(args[0], DynamicContext):
+        if not isinstance(args[1], DynamicContext):
+            return False
+        if args[0].name != args[1].name:
+            return False
+        node1 = DrbQueryFuncUtil.get_node(args[0])
+        node2 = DrbQueryFuncUtil.get_node(args[1])
+
+    elif isinstance(args[1], DynamicContext):
+        return False
+
+    if isinstance(node1, DrbNode):
+        try:
+            if not isinstance(node2, DrbNode):
+                return False
+
+            if node1.name != node2.name:
+                return False
+
+            if len(node1.children) != len(node2.children):
+                return False
+            for index in range(len(node1.children)):
+                child1 = node1[index]
+                if not deep_equal(func,
+                                  current_node,
+                                  static_context,
+                                  child1,
+                                  node2[index]):
+                    return False
+
+            if len(node1.attributes) != len(node2.attributes):
+                return False
+            for attribute_key in node1.attributes.keys():
+                if not deep_equal(func, current_node, static_context,
+                                  node1.get_attribute(attribute_key[0],
+                                                      attribute_key[1]),
+                                  node2.get_attribute(attribute_key[0],
+                                                      attribute_key[1])):
+                    return False
+        except Exception:
+            return False
+    elif isinstance(node2, DrbNode):
+        return False
+    return True
+
+
 def extend_arg_from_list(*args):
     list_arg = []
     for arg_item in args:
@@ -756,21 +898,21 @@ def xs_boolean(func, current_node, static_context, *args):
         return value
 
     if isinstance(value, (int, float, Decimal)):
-        return value == 0
+        return value > 0
 
     if isinstance(value, str):
-        if value.lower() == 'false':
+        if value.lower().strip() == 'false':
             return False
-        if value.lower() == 'true':
+        if value.lower().strip() == 'true':
             return True
         try:
-            return float(value) == 0
+            return float(value) > 0
         except Exception:
             raise DynamicException(ErrorXQUERY.FORG0001, "unable to cast " +
-                                   args[0] + " into boolean")
+                                   str(args[0]) + " into boolean")
 
     raise DynamicException(ErrorXQUERY.FORG0001, "unable to cast " +
-                           args[0] + " into boolean")
+                           str(args[0]) + " into boolean")
 
 
 def xs_decimal(func, current_node, static_context, *args):
@@ -803,6 +945,36 @@ def xs_node(func, current_node, static_context, *args):
     return DrbQueryFuncUtil.get_string(args[0])
 
 
+def text(func, current_node, static_context, *args):
+    check_arg(func, 0, *args)
+
+    value = DrbQueryFuncUtil.get_value(current_node)
+    if isinstance(value, str):
+        return value
+    else:
+        return None
+
+
+def node(func, current_node, static_context, *args):
+    check_arg(func, 0, *args)
+
+    value = DrbQueryFuncUtil.get_node(current_node)
+    if not isinstance(value, DrbNode):
+        return None
+    if static_context.is_in_predicate:
+        return current_node
+
+    result = []
+    index = 0
+    for child in value.children:
+        if isinstance(DrbQueryFuncUtil.get_node(child), DrbNode):
+            node = DynamicContext(child, index)
+            index = index + 1
+            result.append(node)
+
+    return result
+
+
 def xs_anyAtomicType(func, current_node, static_context, *args):
     check_arg(func, 1, *args)
 
@@ -810,11 +982,27 @@ def xs_anyAtomicType(func, current_node, static_context, *args):
     return value
 
 
+def xs_untypedAtomic(func, current_node, static_context, *args):
+    check_arg(func, 1, *args)
+
+    if isinstance(args[0], list):
+        raise DynamicException(ErrorXQUERY.FOAP0001,
+                               "xs_untypedAtomic : requires atomic")
+    value = DrbQueryFuncUtil.get_value(args[0])
+    return value
+
+
 def xs_datetime(func, current_node, static_context, *args):
     check_arg(func, 1, *args)
 
     if isinstance(args[0], datetime.datetime):
         return args[0]
+    try:
+        dt = parse(args[0])
+        return dt
+    except Exception:
+        pass
+
     try:
         if '.' in args[0]:
             return datetime.datetime. \
@@ -912,6 +1100,7 @@ class DrbQueryFuncCall:
         'true': return_true,
         'false': return_false,
         'not': return_not,
+        'boolean': return_boolean,
         # TODO
         'index': get_position,
         # TODO
@@ -929,8 +1118,14 @@ class DrbQueryFuncCall:
         'replace': string_replace,
         'tokenize': string_tokenize,
 
+        'upper-case': upper_case,
+        'lower-case': lower_case,
+
         'exists': exists,
         'empty': empty,
+        'reverse': reverse,
+        'distinct-values': distinct_value,
+        'deep-equal': deep_equal,
         'avg': average,
         'sum': f_sum,
         'min': xquery_min,
@@ -943,6 +1138,8 @@ class DrbQueryFuncCall:
         'data': data,
         'current-dateTime': current_date_time,
         'current-time': current_time,
+        'text': text,
+        'node': node,
 
     }
 
@@ -976,7 +1173,9 @@ class DrbQueryFuncCall:
         'nonNegativeInteger': xs_nonNegativeInteger,
         'nonPositiveInteger': xs_nonPositiveInteger,
         'positiveInteger': xs_positiveInteger,
-        'anyAtomicType': xs_anyAtomicType
+        'anyAtomicType': xs_anyAtomicType,
+        'untypedAtomic': xs_untypedAtomic
+
     }
 
     xs_instance_func = {
