@@ -34,6 +34,7 @@ class DrbQueryVisitor(XQueryParserVisitor):
                  tokens=None):
 
         self.is_module = is_module
+
         self.prefix_module = ''
         self.tokens = tokens
 
@@ -173,6 +174,16 @@ class DrbQueryVisitor(XQueryParserVisitor):
             self.static_context.add_namespace(prefix, full_name)
         return ''
 
+    def visitDefaultNamespaceDecl(
+            self, ctx: XQueryParser.DefaultNamespaceDeclContext):
+        if ctx.stringLiteral() is not None:
+            namespace = self.visitStringLiteral(ctx.stringLiteral())
+            if ctx.KW_ELEMENT():
+                self.static_context.namespace_default_elt = namespace
+            if ctx.KW_FUNCTION():
+                self.static_context.namespace_default_func = namespace
+        return ''
+
     def visitVarDecl(self, ctx: XQueryParser.VarDeclContext):
         varname_full = self.visitVarName(ctx.varName())
         var_namespace, var_name = DrbQueryFuncUtil.split_namespace_name(
@@ -217,6 +228,8 @@ class DrbQueryVisitor(XQueryParserVisitor):
 
     def visitSequenceType(self, ctx: XQueryParser.SequenceTypeContext):
         res = self.visitItemType(ctx.itemType())
+        if res is None:
+            return res
         if ctx.STAR():
             # to simplify if max > 1 like 100 there are no limit
             return res, 0, 100
@@ -230,8 +243,13 @@ class DrbQueryVisitor(XQueryParserVisitor):
 
     def visitArgumentList(self, ctx: XQueryParser.ArgumentListContext):
         result = []
+        list_context = self.list_context
+        dynamic_context = self.dynamic_context
         for arg in ctx.argument():
             result.append(self.visitArgument(arg))
+            self.list_context = list_context
+            self.dynamic_context = dynamic_context
+
         return result
 
     @staticmethod
@@ -443,44 +461,45 @@ class DrbQueryVisitor(XQueryParserVisitor):
             list_right = [right]
 
         test_ok = False
+
         for left_item in list_left:
             for right_item in list_right:
+                if op == 'is':
+                    if left_item is right_item:
+                        test_ok = True
+                        break
+                    else:
+                        continue
+
+                left_value, right_value = \
+                    DrbQueryFuncUtil.convert_before_compare(left_item,
+                                                            right_item)
+
                 if op == 'eq' or op == '=':
-                    if DrbQueryFuncUtil.get_value(left_item) == \
-                            DrbQueryFuncUtil.get_value(right_item) \
-                            or str(DrbQueryFuncUtil.get_value(left_item)) \
-                            == str(DrbQueryFuncUtil.get_value(right_item)):
+                    if DrbQueryFuncUtil.is_equal(left_value, right_value):
                         test_ok = True
                         break
                 elif op == 'ne' or op == '!=':
-                    if DrbQueryFuncUtil.get_value(left_item) != \
-                            DrbQueryFuncUtil.get_value(right_item) \
-                            or str(DrbQueryFuncUtil.get_value(left_item)) \
-                            != str(DrbQueryFuncUtil.get_value(right_item)):
+                    if left_value != right_value:
                         test_ok = True
                         break
                 elif op == 'gt' or op == '>':
-                    if DrbQueryFuncUtil.get_value(left_item) > \
-                            DrbQueryFuncUtil.get_value(right_item):
+                    if left_value > right_value:
                         test_ok = True
                         break
                 elif op == 'ge' or op == '>=':
-                    if DrbQueryFuncUtil.get_value(left_item) >= \
-                            DrbQueryFuncUtil.get_value(right_item):
+                    if left_value >= right_value:
                         test_ok = True
                         break
                 elif op == 'lt' or op == '<':
-                    if DrbQueryFuncUtil.get_value(
-                            left_item) < \
-                            DrbQueryFuncUtil.get_value(right_item):
+                    if left_value < right_value:
                         test_ok = True
                         break
                 elif op == 'le' or op == '<=':
-                    if DrbQueryFuncUtil.get_value(
-                            left_item) <= \
-                            DrbQueryFuncUtil.get_value(right_item):
+                    if left_value <= right_value:
                         test_ok = True
                         break
+
             if test_ok:
                 break
 
@@ -654,6 +673,7 @@ class DrbQueryVisitor(XQueryParserVisitor):
                                ctx: XQueryParser.AbbrevReverseStepContext):
         if ctx.DDOT() is not None:
             self.current_axe.is_parent = True
+
         return ''
 
     def visitAbbrevForwardStep(self,
@@ -710,10 +730,14 @@ class DrbQueryVisitor(XQueryParserVisitor):
         left = self.visitAdditiveExpr(ctx.additiveExpr(0))
         right = self.visitAdditiveExpr(ctx.additiveExpr(1))
 
+        left = DrbQueryFuncUtil.get_numeric_value(left)
+        right = DrbQueryFuncUtil.get_numeric_value(right)
+
         res = list(range(left, right + 1))
         return res
 
     def visit_the_list_of_predicate(self,  predicate_list, context_list):
+        self.static_context.is_in_predicate = True
         result = context_list
         for predicate_exp in predicate_list:
             res_predicate = self.visitPredicateContext(predicate_exp,
@@ -724,6 +748,7 @@ class DrbQueryVisitor(XQueryParserVisitor):
                 context_list, res_predicate, result)
 
             context_list = result
+        self.static_context.is_in_predicate = False
         return result
 
     def visitCommonContent(self, ctx: XQueryParser.CommonContentContext):
@@ -865,12 +890,8 @@ class DrbQueryVisitor(XQueryParserVisitor):
         (namespace, name) = DrbQueryFuncUtil. \
             split_namespace_name(q_name)
 
-        if namespace is not None:
-            namespace = self.get_namespace_full_name(namespace)
-        elif len(self.dynamic_context.namespace_default_elt) > 0:
-            namespace = self.dynamic_context.namespace_default_elt
-
-        node = DrbXqueryItem(None, name, namespace)
+        namespace_full = None
+        node = DrbXqueryItem(None, name, namespace, namespace_full)
         self.dynamic_context = DynamicContext(
             node, dynamic_context_prev=self.dynamic_context)
 
@@ -879,6 +900,12 @@ class DrbQueryVisitor(XQueryParserVisitor):
                 ctx.dirAttributeList(), node)
             node.set_attribute(attributes_node)
 
+        if namespace is not None:
+            node._namespace_uri = self.get_namespace_full_name(namespace)
+        elif node.namespace_uri is None and \
+                len(self.dynamic_context.namespace_default_elt) > 0:
+            node._namespace_uri = self.dynamic_context.namespace_default_elt
+
         return node
 
     def visitDirElemConstructorOpenClose(self, ctx: XQueryParser.
@@ -909,6 +936,8 @@ class DrbQueryVisitor(XQueryParserVisitor):
             value_content = None
             for item_value in result_value:
                 node.order_elt.append(item_value)
+                if isinstance(item_value, DrbNode):
+                    node.append_child(item_value)
                 if isinstance(item_value, DynamicContext):
                     if item_value.is_attribute():
                         node.add_attribute(item_value.name,
@@ -953,8 +982,10 @@ class DrbQueryVisitor(XQueryParserVisitor):
 
     def visitDirectConstructor(self, ctx: XQueryParser.
                                DirectConstructorContext):
-        # self.dynamic_context.namespace_default_elt = \
-        #     self.static_context.namespace_default_elt
+        if self.dynamic_context.namespace_default_elt is None or \
+                len(self.dynamic_context.namespace_default_elt) == 0:
+            self.dynamic_context.namespace_default_elt = \
+                self.static_context.namespace_default_elt
 
         if ctx.dirElemConstructorSingleTag() is not None:
             result = self.visitDirElemConstructorSingleTag(
@@ -963,8 +994,11 @@ class DrbQueryVisitor(XQueryParserVisitor):
             result = self.visitDirElemConstructorOpenClose(
                 ctx.dirElemConstructorOpenClose())
         else:
+            if ctx.getText().startswith('<!--') and\
+                    ctx.getText().endswith('-->'):
+                return None
             raise StaticException(ErrorXQUERY.XPST0003,
-                                  "unable to find constructor")
+                                  "unable to find constructor" + ctx.getText())
 
         return result
 
@@ -1046,10 +1080,11 @@ class DrbQueryVisitor(XQueryParserVisitor):
 
         result = []
 
+        list_context = self.list_context
         for single_exp in ctx.exprSingle():
             res_partial = self.visitExprSingle(single_exp)
             self.append_or_extend(res_partial, result)
-
+            self.list_context = list_context
         return result
 
     # Visit a parse tree produced by XQueryParser#parenthesizedExpr.
@@ -1064,6 +1099,7 @@ class DrbQueryVisitor(XQueryParserVisitor):
             return self.visitChildren(ctx)
 
         result = self.visitPrimaryExpr(ctx.primaryExpr())
+
         predicate_list = ctx.predicate()
         if predicate_list is not None and len(predicate_list) > 0:
             if isinstance(result, list):
@@ -1189,16 +1225,23 @@ class DrbQueryVisitor(XQueryParserVisitor):
                 node = context
             next_nodes = []
 
+            if self.current_axe.is_parent:
+                if isinstance(node, DrbNode):
+                    parent = node.parent
+                else:
+                    parent = context.parent
+
+                if parent is not None:
+                    list_result.append(DynamicContext(
+                        parent,
+                        position=len(list_result) + 1))
+                continue
+
             if not isinstance(node, DrbNode):
                 raise DynamicException(ErrorXQUERY.XPTY0020, "Error ")
 
             try:
-                if self.current_axe.is_parent:
-                    list_result.append(DynamicContext(
-                        node.parent,
-                        position=len(list_result) + 1))
-
-                elif self.current_axe.navigation_step == NavigationStep.\
+                if self.current_axe.navigation_step == NavigationStep.\
                         DESCENDANT_OR_SELF:
                     if self.current_axe.is_wildcard or \
                             self.current_axe.name == node.name and \
@@ -1264,10 +1307,11 @@ class DrbQueryVisitor(XQueryParserVisitor):
                         tokens=self.tokens)
                     result_partial = visitor_exp.visitStepExpr(exp)
                     index = index + 1
-                    if isinstance(result_partial, list):
-                        result.extend(result_partial)
-                    else:
-                        result.append(result_partial)
+                    if result_partial is not None:
+                        if isinstance(result_partial, list):
+                            result.extend(result_partial)
+                        else:
+                            result.append(result_partial)
 
                 self.list_context = result
         return result
@@ -1306,7 +1350,7 @@ class DrbQueryVisitor(XQueryParserVisitor):
         if namespace is not None:
             namespace = self.get_namespace_full_name(namespace)
 
-        return DynamicContext(node=name, value=value_attr,
+        return DynamicContext((name, namespace), value=value_attr,
                               namespace=namespace,
                               attribute=True)
 
@@ -1326,11 +1370,12 @@ class DrbQueryVisitor(XQueryParserVisitor):
                                   ctx.getText())
 
         (namespace, name) = DrbQueryFuncUtil.split_namespace_name(name)
+        namespace_full = None
         if namespace is not None:
-            namespace = self.get_namespace_full_name(namespace)
+            namespace_full = self.get_namespace_full_name(namespace)
         elif len(self.dynamic_context.namespace_default_elt) > 0:
-            namespace = self.dynamic_context.namespace_default_elt
-        node = DrbXqueryItem(None, name, namespace)
+            namespace_full = self.dynamic_context.namespace_default_elt
+        node = DrbXqueryItem(None, name, namespace, namespace_full)
 
         if ctx.enclosedContentExpr() is not None:
             result = self.visitEnclosedContentExpr(ctx.enclosedContentExpr())
@@ -1402,6 +1447,7 @@ class DrbQueryVisitor(XQueryParserVisitor):
         if self.flwor_scope_list is None:
             return self.visitChildren(ctx)
 
+        list_context = self.list_context
         var_name = self.visitVarName(ctx.varName())
 
         # TODO Type declaration
@@ -1418,11 +1464,13 @@ class DrbQueryVisitor(XQueryParserVisitor):
         for scope in self.flwor_scope_list:
             self.static_context.in_scope_variables = scope
             ret_partial = self.visitExprSingle(ctx.exprSingle())
+            key_name = self.static_context.get_key_var(var_name)
+
             if isinstance(ret_partial, list):
                 index = 1
                 for item in ret_partial:
                     current_scope = scope.copy()
-                    current_scope[None, var_name] = XqueryVariable(
+                    current_scope[key_name] = XqueryVariable(
                         var_name,
                         value=item,
                         type_var=type_var)
@@ -1433,12 +1481,15 @@ class DrbQueryVisitor(XQueryParserVisitor):
                     index = index + 1
             else:
                 current_scope = scope.copy()
-                current_scope[None, var_name] = XqueryVariable(
+                current_scope[key_name] = XqueryVariable(
                     var_name,
                     value=ret_partial,
                     type_var=type_var)
                 scope_new.append(current_scope)
+            self.list_context = list_context
+
         self.flwor_scope_list = scope_new
+
         return ''
 
     def visitWhereClause(self, ctx: XQueryParser.WhereClauseContext):
@@ -1458,12 +1509,13 @@ class DrbQueryVisitor(XQueryParserVisitor):
     def visitFlworExpr(self, ctx: XQueryParser.FlworExprContext):
         old_scop = self.static_context.in_scope_variables
         old_context = self.list_context
+        old_scope_list = self.flwor_scope_list
 
         self.flwor_scope_list = [old_scop.copy()]
 
         res = self.visitChildren(ctx)
 
-        self.flwor_scope_list = None
+        self.flwor_scope_list = old_scope_list
         self.static_context.in_scope_variables = old_scop
         self.list_context = old_context
         return res
