@@ -483,20 +483,17 @@ class TestDrbXQuery(unittest.TestCase):
         self.assertEqual(len(node_result), 1)
         self.assertEqual(node_result[0], 21)
 
-    def test_xquery_progess(self):
+    def test_xquery_is(self):
 
-        self.node_file = DrbFileFactory().create(self.xml_file)
-        self.node = XmlNodeFactory().create(self.node_file)
+        query = "declare variable $a := <a/>;" \
+                "\ndeclare function local:testDoubleNodeIdentity(" \
+                "$a as node(), " \
+                "$b as node())" \
+                "\n{\n$a is $b\n};" \
+                "\nlocal:testDoubleNodeIdentity( <a/>, <a/>)," \
+                "\nlocal:testDoubleNodeIdentity($a, $a)"
+        drb_query = DrbXQuery(query)
 
-        # query = DrbXQuery('string(<a attr="content"><!-- NOTINC -->1<b>2<c><'
-        #                   '!-- NOTINC -->34</c><!-- NOTINC -->56</b>7</a>) ')
-        query = DrbXQuery('empty(abs(()))')
+        node_result = drb_query.execute(None)
 
-        list_var = {'x': 9, 'y': 12}
-
-        node_result = query.execute(self.node, list_var)
-
-        print(len(node_result))
-
-        for res in node_result:
-            print(DrbQueryFuncUtil.get_string(res))
+        self.assertListEqual(node_result, [False, True])
